﻿using BenchmarkDotNet.Attributes;
using BenchmarkDotNet.Running;
using System.Runtime.CompilerServices;
using System.Text;

BenchmarkRunner.Run<InterpolatedStringBenchmarks>();

var b = new InterpolatedStringBenchmarks { A = 10, B = 20, C = 30, D = 40 };
Console.WriteLine(b.Concat());
Console.WriteLine(b.Interpolated());
Console.WriteLine(b.StringBuilder());
Console.WriteLine(b.StringFormat());

[MemoryDiagnoser]
public class InterpolatedStringBenchmarks {
	[Params(10)]
	public int A;

	[Params(20)]
	public int B;

	[Params(30)]
	public int C;

	[Params(40, 123_456_789)]
	public int D;

	[Benchmark]
	public string Concat() {
		return A.ToString() + " and " + B.ToString() + " with " + C.ToString() + " then " + D.ToString();
	}

	[Benchmark]
	public string Interpolated() {
		return $"{A} and {B} with {C} then {D}";
	}

	[Benchmark]
	public string StringBuilder() {
		var sb = new StringBuilder();
		sb.Append(A).Append(" and ").Append(B).Append(" with ").Append(C).Append(" then ").Append(D);
		return sb.ToString();
	}

	[Benchmark]
	public string StringFormat() {
		return string.Format("{0} and {1} with {2} then {3}", A, B, C, D);
	}
}